// -*- C++ -*-
//
// user_ana.cc
// Template source file
//
// Author:  Mitsuhiro YAMAGA
// Created: Mon Oct 14 15:21:45 JST 2002
//
// $Id: user_ana.cc,v 1.1 2002/10/14 08:03:24 yamaga Exp $
//
// Revision History
//
// $Log: user_ana.cc,v $
// Revision 1.1  2002/10/14 08:03:24  yamaga
// new example
//
//
//

#include <iostream>
#include <iomanip>
#include "E391Detector/E391Detector.h"
#include "TupleManager/TupleManager.h"
#include "TupleManager/TMNtuple.h"

extern TupleManager  otuple;
extern const E391Detector&  e391det;


// User defined Ntuples
TMNtuple*  ntEvent;
TMNtuple*  ntScaler;

#define  NCSI  576
typedef struct{
  int  runno,spill,evtno,trig;
  int  ncsi;
  int  csitdc[NCSI];
  int  csiadc[NCSI];
  int  csiadcrg[NCSI];
} EVENTDATA;
EVENTDATA  eventdata;
EVENTDATA  eventdata_zero;

static char*  s_eventdata =
"runno:I, spill:I, evtno:I, trig:I, "
"ncsi:I::[0,576], "
"csitdc(ncsi):I::[-1,4095], "
"csiadc(ncsi):I::[-1,4095], "
"csiadcrg(ncsi):I::[0,1] "
"";


// init routine
void
user_init()
{
  // Histogram definition
  ntEvent = otuple.ntuple( 1,"Event Data","ADCTDC",
                           &eventdata,s_eventdata );
  
}


// begin_of_run routine
void
user_begin_of_run( int run_number )
{
  // 
}


// process_event routine
void
user_process_event( int run_number, int spill_number, int event_number )
{
  static int  ct = 0;
  if ( ++ct % 100 == 0 ){
    std::cout << "count=" << ct << " Run" << run_number 
              << " spill=" << spill_number << " Event " << event_number
              << std::endl;
  }

  
  if ( event_number > 0 ){ 
    // Event Data
    if ( e391det.adcCrate(1).valid() ){
      int  trig = e391det.adcCrate(1).trg() & 0xffff;
      
      eventdata.runno = run_number;
      eventdata.spill = spill_number;
      eventdata.evtno = event_number;
      eventdata.trig  = trig;
      eventdata.ncsi  = NCSI;
      for( int i=0;i<NCSI;i++ ){
        int  tdc   = e391det.CsI(i).tdc();
        int  adc   = e391det.CsI(i).adc();
        int  range = (adc & 0x00010000) >> 16;
        eventdata.csitdc[i]   = (tdc >= 0) ? (tdc & 0xffff) : -1;
        eventdata.csiadc[i]   = (adc >= 0) ? (adc & 0xffff) : -1;
        eventdata.csiadcrg[i] = range;
      }
      ntEvent->dump();
      eventdata = eventdata_zero;
    }
  }

  else {
    // Scaler Data
  }

}


// end_of_run routine
void
user_end_of_run( int run_number )
{
  //
}


// term routine
void
user_term()
{
  //
}


