// -*- C++ -*-
//
// eventBuilder.cc
//
// Author:  Mitsuhiro YAMAGA
// Created: Sun Aug 11 22:09:49 JST 2002
//
// $Id: eventBuilder.cc,v 1.4 2002/12/15 15:20:10 yamaga Exp $
//
// Revision History
//
// $Log: eventBuilder.cc,v $
// Revision 1.4  2002/12/15 15:20:10  yamaga
// Added option -ftd0slip to handle the event shift of FTD bank.
//
// Revision 1.3  2002/11/13 13:06:50  yamaga
// Slip-buffer increased to 1000. -maxslip option added.
//
// Revision 1.2  2002/09/20 14:20:24  yamaga
// option added for event_number
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <list>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include "MidasManager/MidasBank.h"
#include "MidasManager/MidasFileIO.h"


// globals
MidasBank    imbank;
MidasBank    ombank;
MidasFileIO  ofile;
int  Debug = 0;
int  MaxSlip = 1000;

int  A0evnumSlot = 18;
int  A1evnumSlot = 18;
int  T0evnumSlot = 16;
int  T1evnumSlot = 17;
int  A0evnumChannel = 88;
int  A1evnumChannel = 88;
int  T0evnumChannel = 56;
int  T1evnumChannel = 56;

int  Ftd0Slip = 0;
int  Ftd1Slip = 0;


//////// user code is defined in the user_ana.cc
extern void  user_init();
extern void  user_begin_of_run( int runnum );
extern int   user_process_event( EVENT_HEADER* pheader,void* pevent,int mask);
extern void  user_end_of_run( int runnum );
extern void  user_term();


////////
// usage
static void
usage( int argc, char* argv[] )
{
  std::cout << "usage : " << argv[0] << " [options]"
            << std::endl 
            << "  options:" << std::endl
            << "    -i <filenames...> (you can use %05d for run_number."
            << " e.g. run%05d.mid)"
            << std::endl 
            << "    -r <run#from> <run#to> (work with %05d)"
            << std::endl
            << "    -n <#event>"
            << std::endl 
            << "    -skip <skipevent>"
            << std::endl
            << "    -mask <maskbit> (bit5-0 = FTD1 FDT0 ADC1 ADC0 TDC1 TDC0)"
            << std::endl
            << "    -o <filename> (you can use %05d for run_number."
            << " e.g. build%05d.mid)"
            << std::endl
            << "    -en_adc0 <slot> <ch>" << std::endl
            << "    -en_adc1 <slot> <ch>" << std::endl
            << "    -en_tdc0 <slot> <ch>" << std::endl
            << "    -en_tdc1 <slot> <ch> (event_number input channel)" 
            << std::endl
            << "    -ftd0slip <ev> " << std::endl
            << "    -ftd1slip <ev> (# of slipped events for FTD0,FTD1)"
            << std::endl
            << "    -maxslip <max> (maximum buffer size to hold slipped events)" 
            << std::endl
            << std::endl;
}


////////
// main
int
main( int argc, char* argv[] )
{
  if ( argc < 2 ){
    usage( argc,argv );
    return( 1 );
  }

  // command parser
  int  Skipevent = 0;
  int  Mask    = 0;
  int  Nevent  = -1;
  int  runFrom = 0;
  int  runTo   = 0;
  std::list<std::string>  iflist;
  std::list<std::string>  iftmp;
  std::string  ofname;
  for( int i=1;i<argc;i++ ){
    if ( argv[i][0] == '-' ){
      if ( strcmp( argv[i],"-i" ) == 0 ){
        while( ++i<argc && argv[i][0] != '-' ){
          iftmp.push_back( argv[i] );
        }
        i--;
      }
      else if ( strcmp( argv[i],"-r" ) == 0 ){
        runFrom =  strtol( argv[++i],NULL,0 );
        runTo   =  strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-n" ) == 0 ){
        Nevent = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-skip" ) == 0 ){
        Skipevent = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-mask" ) == 0 ){
        Mask = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-en_adc0" ) == 0 ){
        if ( ++i < argc && argv[i][0] != '-' ){
          A0evnumSlot = strtol( argv[i],NULL,0 );
          if ( ++i < argc && argv[i][0] != '-' )
            A0evnumChannel = strtol( argv[i],NULL,0 );
          else
            i--;
        }
        else
          i--;
      }
      else if ( strcmp( argv[i],"-en_adc1" ) == 0 ){
        if ( ++i < argc && argv[i][0] != '-' ){
          A1evnumSlot = strtol( argv[i],NULL,0 );
          if ( ++i < argc && argv[i][0] != '-' )
            A1evnumChannel = strtol( argv[i],NULL,0 );
          else
            i--;
        }
        else
          i--;
      }
      else if ( strcmp( argv[i],"-en_tdc0" ) == 0 ){
        if ( ++i < argc && argv[i][0] != '-' ){
          T0evnumSlot = strtol( argv[i],NULL,0 );
          if ( ++i < argc && argv[i][0] != '-' )
            T0evnumChannel = strtol( argv[i],NULL,0 );
          else
            i--;
        }
        else
          i--;
      }
      else if ( strcmp( argv[i],"-en_tdc1" ) == 0 ){
        if ( ++i < argc && argv[i][0] != '-' ){
          T1evnumSlot = strtol( argv[i],NULL,0 );
          if ( ++i < argc && argv[i][0] != '-' )
            T1evnumChannel = strtol( argv[i],NULL,0 );
          else
            i--;
        }
        else
          i--;
      }
      else if ( strcmp( argv[i],"-ftd0slip" ) == 0 ){
        Ftd0Slip = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-ftd1slip" ) == 0 ){
        Ftd1Slip = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-maxslip" ) == 0 ){
        MaxSlip = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-d" ) == 0 ){
        Debug = 1;
      }
      else if ( strcmp( argv[i],"-o" ) == 0 ){
        ofname = argv[++i];
      }
      else {
        usage( argc,argv );
        return( 1 );
      }
    }
    else{
      std::cout << " unknown option : " << argv[i] << std::endl;
      usage( argc,argv );
      return( 1 );
    }
  }
  // iflist
  for( std::list<std::string>::iterator i = iftmp.begin();
       i != iftmp.end(); i++ ){
    if ( strchr( i->c_str(),'%' ) ){
      for( int rn=runFrom; rn<=runTo; rn++ ){
        char  s[1024];
        sprintf( s,i->c_str(),rn );
        iflist.push_back( std::string( s ) );
      }
    }
    else{
      iflist.push_back( *i );
    }
  }
  
  // One output file
  ofile.bank( ombank );
  bool  out_each = true;
  if ( !strchr( ofname.c_str(),'%' ) && !ofname.empty() ){
    out_each = false;
    ofile.open( ofname,O_CREAT|O_EXCL|O_RDWR|O_TRUNC|O_LARGEFILE,0644 );
    if ( ofile.good() )
      std::cout << "Opening output file : " << ofname << std::endl;
    else{
      std::cout << "Cannot open output file : " << ofname << std::endl;
      perror( "open" );
    }
  }

  // call user init routine
  user_init();
  
  // main loop
  for( std::list<std::string>::iterator f = iflist.begin();
       f != iflist.end(); f++ ){
    // open data file
    MidasFileIO  mfile( *f,O_RDONLY|O_LARGEFILE );
    if ( mfile.good() ){
      mfile.bank( imbank );
      std::cout << "Opening input file : " << *f << std::endl;

      // event bank
      EVENT_HEADER*  pheader = imbank.eventHeader();
      void*          pevent  = imbank.eventBuffer();

      // Begin_of_run record
      int  run_number = -1;
      if ( mfile.readBOR() ){
        run_number = pheader->serial_number;
        std::cout << "Run = " << run_number << std::endl;

        // open each output file
        if ( out_each && !ofname.empty() ){
          char  s[1024];
          sprintf( s,ofname.c_str(),run_number );
          ofile.open( s,O_CREAT|O_EXCL|O_RDWR|O_TRUNC|O_LARGEFILE,0644 );
          if ( ofile.good() )
            std::cout << "Opening output file : " << s << std::endl;
          else{
            std::cout << "Cannot open output file : " << s << std::endl;
            perror( "open" );
          }
        }
        // Write BOR
        if ( ofile.good() ){
          ofile.bank( imbank );
          ofile.writeSpecialEvent();
          ofile.bank( ombank );
        }
        
        // call user begin_of_run routine
        user_begin_of_run( run_number );
      }

      // Process Event
      mfile.skipEvent( 0,Skipevent );
      for( int i=0; i<Nevent || Nevent<0; i++ ){
        int  id = mfile.readEvent(0);
        if ( (id & 0x8000) == 0x8000 ){
          // Special event (BOR,EOR,MES)
          if ( ofile.good() ){
            ofile.bank( imbank );
            ofile.writeSpecialEvent();
            ofile.bank( ombank );
          }
          if ( id == EVENTID_BOR ){
            // call user begin_of_run routine
            user_begin_of_run( run_number );
          }
          else if ( pheader->event_id == EVENTID_EOR ){
            // call user end_of_run routine
            user_end_of_run( run_number );
          }
        }
        else if ( id > 0 ){
          if ( user_process_event( pheader,pevent,Mask ) < 0 )
            break;
        }
        else{
          break;
        }
      }

      // Close each output file
      if ( out_each && !ofname.empty() ){
        ofile.close();
      }
    }
    else{
      std::cout << "No such file : " << *f << std::endl;
    }

  }

  // call user term routine
  user_term();

  // close 
  if ( out_each == false && !ofname.empty() ){
    ofile.close();
  }
  
  //
  return( 0 );
}

