// -*- C++ -*-
//
// user_ana.cc
//
// Author:  Ken Sakashita
// Created: Mon Oct  7 22:51:58 JST 2002
//
// $Id: user_ana.cc,v 1.6 2002/12/03 20:55:58 kensh Exp $
//
// $Log: user_ana.cc,v $
// Revision 1.6  2002/12/03 20:55:58  kensh
// fixed minor bug
//
// Revision 1.5  2002/12/03 18:49:53  kensh
// Remove making RWN in online stage.
// Add some histograms in order to get Run_number and Spill_number.
//
// Revision 1.4  2002/11/04 19:08:48  yamaga
// Scaler data for NXcluster corrected.
//
// Revision 1.3  2002/10/31 10:07:41  yamaga
// Use TupleManager and ntuple. Sampling scheme and scaler channel changed.
//
// Revision 1.2  2002/10/18 16:41:08  yamaga
// Fixed the overflow problem.
//
// Revision 1.1  2002/10/07 14:04:50  kensh
//
//
// Add new package "onlineMonitor"
//
//
//
//

#include <iostream>
#include <iomanip>
#include <list>
#include <unistd.h>
#include "E391Detector/E391Detector.h"
#include "ccfortran/ccfortran.h"
#include "TupleManager/TupleManager.h"

// These are defined int the E391Detector.h
//#define  TRIG_SPILLON  0x01
//#define  TRIG_CLOCK    0x02
//#define  TRIG_COSMIC   0x04
//#define  TRIG_XE       0x08
//#define  TRIG_NCLUS    0x10

#define  SCL_SPILL     4
#define  SCL_REQUEST   12
#define  SCL_ACCEPT    13
#define  SCL_SEC       0
#define  SCL_TMON      2
#define  SCL_CLOCK     36
#define  SCL_COSMIC    38
#define  SCL_XE        40
#define  SCL_NCLUS     42
#define  SCL_NXCLUS    44
#define  SCL_SPECIAL1  46
#define  SCL_SPECIAL2  48

//#define  MAX_ADC_CH    96
//#define  MAX_TDC_CH    64

//
extern TupleManager otuple;
extern E391Detector&  e391det;
extern unsigned int  TriggerBit;

//
int AdcCutH  = 580;
int AdcCutL  = 500;

//nnn
// For RunControl
static int oldnRequestEvent;
static int oldnAcceptEvent;
static int oldnSEC;
static int oldnTMON;
//
static int oldnClockEvent;
static int oldnCosmicEvent;
static int oldnXeEvent;
static int oldnNclsEvent;
static int oldnNXclsEvent;
static int oldnSpecial1Event;
static int oldnSpecial2Event;
//
static long long RequestEvent = 0;
static long long AcceptEvent  = 0;

//
// M.Yamaga
static bool  init = false;

// Higz
#define NWPAWC  1000000
int  pawc_[NWPAWC];

//
// User defined Ntuple

void
set_Hbook()
{
  HDELET( 0 );
  
  //
  HBOOK1(  1, "N SEC per spill", 1, 0., 1.);
  HBOOK1(  2, "N TMON per spill", 1, 0., 1.);
  HBOOK1(  3, "N clock per spill", 1, 0., 1.);
  HBOOK1(  4, "N cosmic per spill", 1, 0., 1.);
  HBOOK1(  5, "N Xe per spill", 1, 0., 1.);
  HBOOK1(  6, "N Cluster per spill", 1, 0., 1.);
  HBOOK1(  7, "NX Cluster per spill", 1, 0., 1.);
  HBOOK1(  8, "Trigger efficiency per spill", 1, 0., 1.);
  HBOOK1(  9, "Trigger efficiency averaged", 1, 0., 1.);
  HBOOK1( 10, "Run#", 1, 0., 1.);
  HBOOK1( 11, "Spill", 1, 0., 1.);
    

}

// init routine
void
user_init()
{
  // init HIGZ
  //HLIMIT( NWPAWC );
  //HLIMAP( NWPAWC, "TEST" );
  otuple.initMap( NWPAWC, "ONLN" );
  //
  set_Hbook();

  
}


// begin_of_run routine
void
user_begin_of_run( int run_number )
{
  //
  set_Hbook();
}


// process_event routine
void
user_process_event( int run_number, int spill_number,
                    int event_number, int trigger_bit )
{

}


// process_event routine
void
user_process_event_exp( HNDLE hbuf,HNDLE request_id,
                        EVENT_HEADER* pheader, void* pevent )
{
  /* ODB structures */
  extern RUNINFO  runinfo;
  
  static int  old_spillnum = 0;
  int  spillnum = pheader->serial_number;

  //
  static int  RunNumber = 0;
  if ( RunNumber != runinfo.run_number || spillnum < old_spillnum ){
    init = false;
    RunNumber = runinfo.run_number;
    e391det.setRunNumber( RunNumber );
    // get DB for this run
    e391det.getDB();
    //e391det.getDB( 1 );
    set_Hbook();
    RequestEvent = 0;
    AcceptEvent  = 0;
    //
    HRESET( 10, " ");
    HFILL(  10, 0.5, 0., (float)RunNumber);
  }
  
  //
  e391det.setEvent( pheader, pevent );
  int iSpill = e391det.spillNumber();
  
  // Scaler event
  DWORD*  pdataSCL0;
  int scl0size = e391det.bank( "SCL0", &pdataSCL0 );
  if ( scl0size > 0 ) {
    if ( !init ){
      //initRequestEvent = pdataSCL0[ SCL_REQUEST ];
      //initAcceptEvent  = pdataSCL0[ SCL_ACCEPT  ];
      oldnRequestEvent = pdataSCL0[ SCL_REQUEST ];
      oldnAcceptEvent  = pdataSCL0[ SCL_ACCEPT  ];
      init = true;
    }
    
    //int RequestEvent  = pdataSCL0[ SCL_REQUEST ] - initRequestEvent;
    //int AcceptEvent   = pdataSCL0[ SCL_ACCEPT  ] - initAcceptEvent;
    int nRequestEvent = pdataSCL0[ SCL_REQUEST ] - oldnRequestEvent;
    int nAcceptEvent  = pdataSCL0[ SCL_ACCEPT  ] - oldnAcceptEvent;
    int nSEC          = pdataSCL0[ SCL_SEC     ] - oldnSEC;
    int nTMON         = pdataSCL0[ SCL_TMON    ] - oldnTMON;
    
    int nClockEvent   = pdataSCL0[ SCL_CLOCK   ] - oldnClockEvent;
    int nCosmicEvent  = pdataSCL0[ SCL_COSMIC  ] - oldnCosmicEvent;
    int nXeEvent      = pdataSCL0[ SCL_XE      ] - oldnXeEvent;
    int nNclsEvent    = pdataSCL0[ SCL_NCLUS   ] - oldnNclsEvent;
    int nNXclsEvent   = pdataSCL0[ SCL_NXCLUS  ] - oldnNXclsEvent;

    // overflow?
    if ( nRequestEvent < 0 )
      nRequestEvent += (1<<24);
    if ( nAcceptEvent < 0 )
      nAcceptEvent += (1<<24);
    if ( nSEC < 0 )
      nSEC += (1<<24);
    if ( nTMON < 0 )
      nTMON += (1<<24);
    if ( nClockEvent < 0 )
      nClockEvent += (1<<24);
    if ( nCosmicEvent < 0 )
      nCosmicEvent += (1<<24);
    if ( nXeEvent < 0 )
      nXeEvent += (1<<24);
    if ( nNclsEvent < 0 )
      nNclsEvent += (1<<24);
    if ( nNXclsEvent < 0 )
      nNXclsEvent += (1<<24);
    
    RequestEvent += nRequestEvent;
    AcceptEvent  += nAcceptEvent;

    //
    HRESET( 1, " ");
    HRESET( 2, " ");
    HRESET( 3, " ");
    HRESET( 4, " ");
    HRESET( 5, " ");
    HRESET( 6, " ");
    HRESET( 7, " ");
    HRESET( 8, " ");
    HRESET( 9, " ");
    HRESET( 11, " ");
    
    int  d_spillnum = iSpill - old_spillnum;
    if ( d_spillnum <= 0 )
      d_spillnum = 1;
      
    HFILL(  1, 0.5, 0., (float)nSEC );
    HFILL(  2, 0.5, 0., (float)nTMON );
    HFILL(  3, 0.5, 0., (float)nClockEvent/d_spillnum);
    HFILL(  4, 0.5, 0., (float)nCosmicEvent/d_spillnum);
    HFILL(  5, 0.5, 0., (float)nXeEvent/d_spillnum);
    HFILL(  6, 0.5, 0., (float)nNclsEvent/d_spillnum);
    HFILL(  7, 0.5, 0., (float)nNXclsEvent/d_spillnum);
    if( nRequestEvent != 0 )
      HFILL(  8, 0.5, 0., (float)nAcceptEvent/nRequestEvent );
    if( RequestEvent != 0 )
      HFILL(  9, 0.5, 0., (float)AcceptEvent/RequestEvent );

    HFILL(  11, 0.5, 0., (float)iSpill);

    // DAQ stability 
    if( iSpill%1000 == 0 ) {
      // DAQ status
      HBOOK1(  50, "# of request event per spill", 1000, (float)iSpill, (float)(iSpill + 1000) );
      HBOOK1(  51, "integrated # of request event", 1000, (float)iSpill, (float)(iSpill + 1000) );
      HBOOK1(  52, "# of accept event per spill", 1000, (float)iSpill, (float)(iSpill + 1000) );
      HBOOK1(  53, "integrated # of accept event", 1000, (float)iSpill, (float)(iSpill + 1000) );
      HBOOK1(  54, "Trigger efficiency per spill", 1000, (float)iSpill, (float)(iSpill + 1000) );
      HBOOK1(  55, "Trigger efficiency", 1000, (float)iSpill, (float)(iSpill + 1000) );

    }
    if( !HEXIST(50) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  50, "# of request event per spill", 1000, (float)xmin, (float)xmax );
    }
    if( !HEXIST(51) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  51, "integrated # of request event", 1000, (float)xmin, (float)xmax );
    }
    if( !HEXIST(52) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  52, "# of accept event per spill", 1000, (float)xmin, (float)xmax );
    }
    if( !HEXIST(53) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  53, "integrated # of accept event", 1000, (float)xmin, (float)xmax );
    }
    if( !HEXIST(54) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  54, "Trigger efficiency per spill", 1000, (float)xmin, (float)xmax );
    }
    if( !HEXIST(55) ) {
      int xmin = ( iSpill - iSpill%1000 );
      int xmax = xmin + 1000;
      HBOOK1(  55, "Trigger efficiency", 1000, (float)xmin, (float)xmax );
    }
    HFILL( 50, 0.5 + (float)iSpill, 0., (float)nRequestEvent);
    HFILL( 51, 0.5 + (float)iSpill, 0., (float)RequestEvent);
    HFILL( 52, 0.5 + (float)iSpill, 0., (float)nAcceptEvent);
    HFILL( 53, 0.5 + (float)iSpill, 0., (float)AcceptEvent);
    if( nRequestEvent != 0 ) 
      HFILL( 54, 0.5 + (float)iSpill, 0., (float)nAcceptEvent/(float)nRequestEvent);
    if( RequestEvent != 0 ) 
      HFILL( 55, 0.5 + (float)iSpill, 0., (float)AcceptEvent/(float)RequestEvent);
 
    //////////////////
    //
    oldnRequestEvent = pdataSCL0[ SCL_REQUEST ];
    oldnAcceptEvent  = pdataSCL0[ SCL_ACCEPT  ];
    oldnSEC          = pdataSCL0[ SCL_SEC     ];
    oldnTMON         = pdataSCL0[ SCL_TMON    ];
    //
    oldnClockEvent   = pdataSCL0[ SCL_CLOCK   ];
    oldnCosmicEvent  = pdataSCL0[ SCL_COSMIC  ];
    oldnXeEvent      = pdataSCL0[ SCL_XE      ];
    oldnNclsEvent    = pdataSCL0[ SCL_NCLUS   ];
    oldnNXclsEvent   = pdataSCL0[ SCL_NXCLUS  ];
    //
    old_spillnum = iSpill;
    
  }

  // Data event
  DWORD*  pdata;
  int  nb = e391det.bank( "NGF0",&pdata );
  if ( nb > 0 ){
    std::cout << "Run"      << runinfo.run_number
              << " spill="  << spillnum
              << " nevent=" << pdata[0]
              << std::endl;
    int  nevent = pdata[0];
    DWORD* pev;
    int nn = e391det.bank("TRG1",&pev);
    if( nn == 0 ) 
      return;
    //std::cout << "trig[0] = " << pev[0] << std::endl;
    int nOffSpill = 0;
    for( int i=0; i < nn; i++ ){
      if( pev[i] & TRIG_SPILLON ) 
	break;
      nOffSpill++;
    }
    int  nOnSpill = nevent - nOffSpill;
    int  nskip = (int)(nOnSpill / 4) + 1;
    std::list<int> evList;
    evList.push_back( (int)(nOffSpill/2) );
    
    int iev = nOffSpill + 1;
    while( 1 ) {
      evList.push_back( iev );
      if( (iev += nskip) > nevent )
	break;
    }
    

    std::cout << " nevent = " << nevent
              << " nsample = " << evList.size() << std::endl;
    static int ct = 0;

    for( std::list<int>::iterator m = evList.begin(); m != evList.end(); m++ ){
      if ( e391det.unpack( *m ) == 0 ) {
	ct += (*m);
	int trig = e391det.triggerBit( *m );
	//int trig = 0;
	std::cout << " iev = " << *m  << " trig = 0x" 
                  << std::hex << pev[*m-1] << std::dec << std::endl; 
        //if ( e391det.adcCrate(1).valid() )
	//  trig = e391det.adcCrate(1).trg() & 0xffff;
	//std::cout << "trig = " << trig << std::endl; 

	// Histgram
	if ( trig&TRIG_CLOCK ) { // clock event
	  // NGF0
	  const std::list<AdcModule>& adc0List = e391det.adcCrate( 0 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc0List.begin();
	       m != adc0List.end(); m++ ){
	    int hid = 1100 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "Pedestal : NGF0 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;

	      if( !rangeBit ) HFILL( hid, 0.5 + (float)ch, 0., (float)rawAdc ); 
	    }
	  } 

	  
	}
	else if ( trig&TRIG_COSMIC ) { // Cosmic event
	  // NGF0
	  const std::list<AdcModule>& adc0List = e391det.adcCrate( 0 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc0List.begin();
	       m != adc0List.end(); m++ ){
	    int hid = 2100 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "CosmicTrig : Hit pattern : NGF0 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // NGF1
	  const std::list<AdcModule>& adc1List = e391det.adcCrate( 1 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc1List.begin();
	       m != adc1List.end(); m++ ){
	    int hid = 2200 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "CosmicTrig : Hit pattern : NGF1 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // SMP0
	  const std::list<TdcModule>& tdc0List = e391det.tdcCrate( 0 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc0List.begin();
	       m != tdc0List.end(); m++ ){
	    int hid = 2300 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "CosmicTrig : Hit pattern : SMP0 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }
	  // SMP1
	  const std::list<TdcModule>& tdc1List = e391det.tdcCrate( 1 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc1List.begin();
	       m != tdc1List.end(); m++ ){
	    int hid = 2400 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "CosmicTrig : Hit pattern : SMP1 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }


	}
	else if ( trig&TRIG_XE ) { // Xe event
	  // NGF0
	  const std::list<AdcModule>& adc0List = e391det.adcCrate( 0 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc0List.begin();
	       m != adc0List.end(); m++ ){
	    int hid = 3100 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "XeTrig : Hit pattern : NGF0 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // NGF1
	  const std::list<AdcModule>& adc1List = e391det.adcCrate( 1 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc1List.begin();
	       m != adc1List.end(); m++ ){
	    int hid = 3200 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "XeTrig : Hit pattern : NGF1 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // SMP0
	  const std::list<TdcModule>& tdc0List = e391det.tdcCrate( 0 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc0List.begin();
	       m != tdc0List.end(); m++ ){
	    int hid = 3300 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "XeTrig : Hit pattern : SMP0 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }
	  // SMP1
	  const std::list<TdcModule>& tdc1List = e391det.tdcCrate( 1 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc1List.begin();
	       m != tdc1List.end(); m++ ){
	    int hid = 3400 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "XeTrig : Hit pattern : SMP1 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }



	}
	else if ( trig&TRIG_NCLUS ) { // Nclus event
	  // NGF0
	  const std::list<AdcModule>& adc0List = e391det.adcCrate( 0 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc0List.begin();
	       m != adc0List.end(); m++ ){
	    int hid = 4100 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "NclusterTrig : Hit pattern : NGF0 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // NGF1
	  const std::list<AdcModule>& adc1List = e391det.adcCrate( 1 ).adcList();
	  for( std::list<AdcModule>::const_iterator m = adc1List.begin();
	       m != adc1List.end(); m++ ){
	    int hid = 4200 + m->pa();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "NclusterTrig : Hit pattern : NGF1 : ADC PA = %02d", m->pa());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_ADC_CH; ch++ ) {
	      int rangeBit = m->data( ch )&0xffff0000;
	      int rawAdc   = m->data( ch )&0xffff;
	      int hit      = ( rangeBit ? rawAdc - AdcCutH : rawAdc - AdcCutL);

	      if( hit > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 
	    }

	  }
	  // SMP0
	  const std::list<TdcModule>& tdc0List = e391det.tdcCrate( 0 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc0List.begin();
	       m != tdc0List.end(); m++ ){
	    int hid = 4300 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "NclusterTrig : Hit pattern : SMP0 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }
	  // SMP1
	  const std::list<TdcModule>& tdc1List = e391det.tdcCrate( 1 ).tdcList();
	  for( std::list<TdcModule>::const_iterator m = tdc1List.begin();
	       m != tdc1List.end(); m++ ){
	    int hid = 4400 + m->ma();
	    if( !HEXIST(hid) ) {
	      char str[128];
	      sprintf(str, "NclusterTrig : Hit pattern : SMP1 : TDC MA = %02d", m->ma());
	      HBOOK1( hid, str, 100, 0., 100. );
	    }
	    for( int ch = 0; ch < MAX_TDC_CH; ch++ ) {
	      int rawTdc   = m->data( ch );
	      if( rawTdc > 0 ) HFILL( hid, 0.5 + (float)ch, 0., 1. ); 

	    }

	  }
	  //
	}

      }
      else {
	break;
      }
    } // End of Event Loop
  } // End of Event Data
}


// end_of_run routine
void
user_end_of_run( int run_number )
{
  //
  HDELET( 0 );
}


// term routine
void
user_term()
{
  //
}


