// -*- C++ -*-
//
// user_ana.cc
//
// Author:  Ken Sakashita
// Created: Fri Nov 29 14:53:25 JST 2002
//
// $Id: user_ana.cc,v 1.3 2002/12/06 23:23:38 kensh Exp $
//
// $Log: user_ana.cc,v $
// Revision 1.3  2002/12/06 23:23:38  kensh
// Add new histograms for Multiplicity Logic.
//
// Revision 1.2  2002/12/01 01:10:23  kensh
// Modified "dscaler" to "dscaler/dspillnum" for histograms.
//
// Revision 1.1  2002/11/29 05:56:53  kensh
//
// Initial version
//
//
//
//
//

#include <iostream>
#include <iomanip>
#include <list>
#include <unistd.h>
#include "E391Detector/E391Detector.h"
#include "ccfortran/ccfortran.h"
#include "TupleManager/TupleManager.h"
#include "scalerMonitor/user_inc.h"

//
extern TupleManager otuple;
extern E391Detector&  e391det;
extern unsigned int  TriggerBit;

//
// M.Yamaga
static bool init   = false;

// Higz
#define NWPAWC  1000000
int  pawc_[NWPAWC];

// Scaler
// For RunControl
static float  scaler[12*20];
static float  dscaler[12*20];
static float  oldscaler[12*20];

void
initHbook()
{
  HDELET( 0 );
    
  //
  for( int i = 0; i < nscl; i++ ) {
    int Hid = 8000 + scl[i].ch;
    char str[256];
    sprintf(str, "Scaler : %s", scl[i].name.c_str());
    HBOOK1( Hid, str, 5000, 0., 5000.);

  }
  // additional histograms
  HBOOK1( 8500, "Trigger Efficiency (/spill)", 5000, 0., 5000.);
  HBOOK1( 8501, "Multiplicity Logic : counting rate", 5, 0., 5.);

  
}

// init routine
void
user_init()
{
  // init HIGZ
  //HLIMIT( NWPAWC );
  //HLIMAP( NWPAWC, "TEST" );
  otuple.initMap( NWPAWC, "SCLM" );
  //
  initHbook();
  
}


// begin_of_run routine
void
user_begin_of_run( int run_number )
{
  //
  initHbook();
}


// process_event routine
void
user_process_event( int run_number, int spill_number,
                    int event_number, int trigger_bit )
{

}


// process_event routine
void
user_process_event_exp( HNDLE hbuf,HNDLE request_id,
                        EVENT_HEADER* pheader, void* pevent )
{
  /* ODB structures */
  extern RUNINFO  runinfo;
  
  static int  old_spillnum = 0;
  int  spillnum = pheader->serial_number;

  //
  static int  RunNumber = 0;
  if ( RunNumber != runinfo.run_number || spillnum < old_spillnum ){
    init = false;
    RunNumber = runinfo.run_number;
    e391det.setRunNumber( RunNumber );
    // get DB for this run
    e391det.getDB();
    //e391det.getDB( 1 );
    initHbook();
  }
  
  //
  e391det.setEvent( pheader, pevent );
  int iSpill = e391det.spillNumber();
  
  // Scaler event
  DWORD*  pdataSCL0;
  int scl0size = e391det.bank( "SCL0", &pdataSCL0 );
  if ( scl0size > 0 ) {
    if ( !init ){
      for( int i=0;i<scl0size;i++ ){
        oldscaler[i] = (float)pdataSCL0[i];
        scaler[i]    = 0.0;
      }
      init = true;
    }

    std::cout << "Run " << RunNumber << " Spill = " << iSpill << std::endl;
    
    for( int i=0;i<scl0size;i++ ){
      dscaler[i] = pdataSCL0[i] - oldscaler[i];
      if ( dscaler[i] < 0 )
        dscaler[i] += (1 << 24);
      scaler[i] += dscaler[i];
    }

    //
    int  dspillnum = iSpill - old_spillnum;
    if ( dspillnum <= 0 )
      dspillnum = 1;
    
    for( int i = 0; i < nscl; i++ ) {
      int Hid = 8000 + scl[i].ch;
      HFILL( Hid, 0.5 + (float)iSpill, 0., (float)dscaler[scl[i].ch]/(float)dspillnum );
    }

    // additional histograms
    float nRequestEvent = (float)dscaler[12]/(float)dspillnum;
    float nAcceptEvent  = (float)dscaler[13]/(float)dspillnum;
    if( nRequestEvent > 0.0 ) HFILL( 8500, 0.5 + (float)iSpill, 0., nAcceptEvent/nRequestEvent );
    
    for( int i = 0; i < 5; i++) {
      int sclch = 103 + i;
      HFILL( 8501, 0.5 + (float)i, 0., (float)dscaler[sclch]/(float)dspillnum );
    }

    //
    for( int i=0;i<scl0size;i++ ){
      oldscaler[i] = pdataSCL0[i];
    }
    old_spillnum = iSpill;
    
  }
}


// end_of_run routine
void
user_end_of_run( int run_number )
{
  //
  HDELET( 0 );
}


// term routine
void
user_term()
{
  //
}


