// -*- C++ -*-
//
// caenetd.cc
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu Aug 15 23:59:03 JST 2002
//
// $Id: caenetd.cc,v 1.2 2002/09/10 12:12:05 yamaga Exp $
//
// Revision History
//
// $Log: caenetd.cc,v $
// Revision 1.2  2002/09/10 12:12:05  yamaga
// updates
//
// Revision 1.1  2002/08/29 14:06:26  yamaga
// Initial version.
//
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <strings.h>
#include <unistd.h>
#include <sys/mman.h>
#include <time.h>
#include <signal.h>
#include <sys/wait.h>

#ifdef OS_SUNOS
#include <sys/vme.h>
#include <sys/vui.h>
#endif // OS_SUNOS


//////
static int  DEBUG = 0;

#define  V288BASE  0xff0000

#define  TXBUF         0x0000
#define  RXBUF         0x0000
#define  STATUSREG     0x0002
#define  TXREG         0x0004
#define  RESETREG      0x0006
#define  INTERRUPTREG  0x0008

#define  CN_OK         0xfffe
#define  CN_NOOP       0xffff

#define  CN_BUSY       0xff00

//////
static int  fd24 = -1;
static unsigned short*  map=0;
static int  msize = 0x10;

static int  PORT = 52700;
static int  sock_listen = -1;
static int  sock        = -1;

static bool  DAEMON = true;

//
//
void  closeVME24();
void  unmapVME24( void* ptr,unsigned int size );

////// 
static void
catch_sigint( int signo )
{
  close( sock_listen );
  close( sock );
  //
  unmapVME24( map,msize );
  closeVME24();
  exit(0);
}

////// 
static void
catch_sigchld( int signo )
{
  //printf( "signal %d caught.\n", signo );
  int  status;
  wait(&status);
}

//////
int
openVME24()
{
  if ( ( fd24 = open("/dev/vme24d16",O_RDWR) ) < 0 ){
    perror( "openVME24 : open" );
    exit( -1 );
  }
  return( 0 );
}


//////
void
closeVME24()
{
  if ( fd24 >= 0 )
    close( fd24 );
}


//
unsigned short*
mapVME24( off_t addr,unsigned int size )
{
  void*  ptr;
  
  ptr = mmap( 0, size, PROT_WRITE|PROT_READ,MAP_SHARED, fd24, addr );
  if ( ptr == (caddr_t)-1){
    perror("mmap");
    exit(1);
  }

  if ( DEBUG )
    printf( "mapVME24 : addr=0x%04p size=0x%04x ptr=0x%08p\n",
            addr,size,ptr );

  return( (unsigned short*)ptr );
}


//
void
unmapVME24( void* ptr,unsigned int size )
{
  if ( ptr != (caddr_t)-1 && ptr != 0 ){
    munmap( (char*)ptr,size );
  }
}


//
unsigned short
vmeread( off_t addr )
{
  unsigned short  data;
  
  if ( 0x00 <= addr && addr <= 0x02 ){
    data = map[addr>>1];
    if ( DEBUG )
      printf( "vmeread : addr=0x%04p data=0x%04x\n",
              addr,data );
  }

  return( data );
}


//
unsigned short
vmewrite( off_t addr, unsigned short cmd )
{
  if ( 0x00 <= addr && addr <= 0x08 ){
    map[addr>>1] = cmd;
    if ( DEBUG )
      printf( "vmewrite : addr=0x%04p data=0x%04x\n",
              addr,cmd );
  }

  return( 0 );
}


//////
void
reset_v288()
{
  vmewrite( RESETREG,0 );
  static const struct timespec ts = { 0, 5000000 }; // 5 msec
  nanosleep( &ts,NULL );
  unsigned short  stat;
  do{
    stat = vmeread( STATUSREG );
  } while ( stat!=CN_NOOP && stat!=CN_OK );

  printf( "reset_v288 called.\n" );
}


//////
unsigned short
wait_response()
{
  unsigned short  dt,stat;
  int  ct = 0;
  do{
    // wait
    //static const struct timespec ts = { 0, 10000 }; // 1 usec
    //nanosleep( &ts,NULL );
    //for( int i=0;i<10;i++ ) // 1.0e6 loop takes 100 msec for CPU5V
    //  ; // 10 loop = 1 usec
    if ( ++ct >= 1000000 ){ // 1 sec
      printf( "wait_response : ct=%d\n",ct );
      reset_v288();
      break;
    }
    // read
    dt   = vmeread( RXBUF );
    stat = vmeread( STATUSREG );
    if ( DEBUG )
      printf( "wait_response : data=0x%04x stat=0x%04x\n",dt,stat );
  } while( stat!=CN_OK );
  
  return( dt );
}


//////
int
read_buf( unsigned short* data, int maxsize )
{
  int  n=0;
  unsigned short  ret;
  data[n++] = ret = wait_response();
  if ( ret == 0 ){ // success
    unsigned short  dt,stat;
    while( n<maxsize ){
      dt   = vmeread( RXBUF );
      stat = vmeread( STATUSREG );
      if ( DEBUG )
        printf( "read_buf : data=0x%04x stat=0x%04x\n",dt,stat );
      if ( stat==CN_NOOP ) // end of data
        break;
      data[n++] = dt;
    }
    return( n );
  }
  else{
    if ( ret != CN_BUSY ){
      time_t  t = time( NULL );
      char  ts[64];
      strcpy( ts,ctime( &t ) );
      ts[strlen(ts)-1] = '\0';
      printf( "%s : read_buf : error=0x%04x\n",
              ts, ret );
    }
  }

  return( n );
}


//////
bool
write_buf( unsigned short data )
{
  unsigned int  stat;
  vmewrite( TXBUF,data );
  while(1) {
    stat = vmeread( STATUSREG );
    if ( DEBUG )
      printf( "write_buf : data=0x%04x stat=0x%04x\n",data,stat );
    if ( stat==CN_NOOP || stat==CN_OK )
      break;
  }

  return( (stat==CN_OK) ? true:false );
}


//////
bool
tx_start()
{
  vmewrite( TXREG,0 );
  unsigned int  stat;
  while ( 1 ) {
    stat = vmeread( STATUSREG );
    if ( DEBUG )
      printf( "tx_start : stat=0x%04x\n",stat );
    if ( stat==CN_NOOP || stat==CN_OK )
      break;
  }

  return( (stat==CN_OK) ? true:false );
}


//////
bool
set_interrupt( unsigned short vec )
{
  vmewrite( INTERRUPTREG,vec );
  unsigned int  stat;
  while ( 1 ) {
    stat = vmeread( STATUSREG );
    if ( DEBUG )
      printf( "set_interrupt : stat=0x%04x\n",stat );
    if ( stat==CN_NOOP || stat==CN_OK )
      break;
  }

  return( (stat==CN_OK) ? true:false );
}


//////
int
process_command( int sock )
{
  char   buf[2000];
  unsigned short*  sbuf = (unsigned short*)buf;
  FILE* fpr = fdopen( sock,"r" );
  FILE* fpw = fdopen( sock,"w" );
  
  // reset
  reset_v288();

  // start 
  while( 1 ){
    // read command from client
    fgets( buf,2000,fpr );
    int  n = strlen( buf );
    buf[n] = 0; // change \n to \0
    if ( DEBUG )
      printf( "read size=%d data=%s\n",n,buf );
    
    char*  p = strtok( buf," ,\t\n" );
    if ( p[0] == 'q' ){ // quit : terminate
      fclose( fpw );
      fclose( fpr );
      return( 0 );
    }
    
    // write commmand to V288
    // command sequence :
    //  controller# 
    //  crate#
    //  commands...
    
    // controller#
    unsigned short  d = strtol( p,NULL,0 );
    if ( write_buf( d ) == false ){
      printf( "process_command : send to V288 failed 1 : data=0x%x\n",d );
    }
    // crate#
    p = strtok( NULL," ,\t\n" );
    d = strtol( p,NULL,0 );
    if ( write_buf( d ) == false ){
      printf( "process_command : send to V288 failed 2 : data=0x%x\n",d );
    }
    // commands...
    p = strtok( NULL," ,\t\n" );
    d = strtol( p,NULL,0 );
    if ( d == 0x19 ){ // 0x19 : change the channel-name
      if ( write_buf( d ) == false ){
        printf( "process_command : send to V288 failed 3 : data=0x%x\n",d );
      }
      // channel#
      p = strtok( NULL," ,\t\n" );
      d = strtol( p,NULL,0 );
      if ( write_buf( d ) == false ){
        printf( "process_command : send to V288 failed 4 : data=0x%x\n",d );
      }
      // channel name
      p = strtok( NULL," ,\t\n" );
      char chname[] = "\0\0\0\0\0\0\0\0\0\0\0\0";
      strncpy( chname,p,11 );
      unsigned short*  ps = (unsigned short*)p;
      for( int i=0;i<6; i++ ){
        if ( write_buf( ps[i] ) == false ){
          printf( "process_command : send to V288 failed 5 : data=0x%x\n",d );
        }
      }
    }
    else {
      do{
        d = strtol( p,NULL,0 );
        if ( write_buf( d ) == false ){
          printf( "process_command : send to V288 failed 6 : data=0x%x\n",d );
        }
      } while( p = strtok( NULL," ,\t\n" ) );
    }
    
    // start transmission
    if ( tx_start() == false ){
      printf( "process_command : transmission failed\n" );
    }
    
    // read response from V288
    n = read_buf( sbuf,1000 );
    if ( sbuf[0] != 0 && sbuf[0] != CN_BUSY ){
      printf( "process_command : read from V288 failed\n" );
    }
    
    // send data to client
    if ( n > 0 ){
      int  wn = fwrite( sbuf,sizeof(short),n,fpw );
      fflush( fpw );
      if ( DEBUG )
        printf( "write size = %d\n", wn );
      if ( wn < n ){
        printf( "process_command : failed to send data to client\n" );
      }
    }
    //const char*  flushstr = "\n";
    //write( sock,flushstr,strlen(flushstr) );
  }

  fclose( fpw );
  fclose( fpr );
  return( 0 );
}


//////
void
usage()
{
  printf( "usage : caenetd [-p port] [-h|--help] [-d(ebug)] [-nodaemon]\n" );
}


//////
int
main( int argc, char* argv[] )
{
  // command argument
  if ( argc > 1 ){
    for( int i=1;i<argc;i++ ){
      if ( strcmp( argv[i],"-h" ) == 0 || strcmp( argv[i],"--help" ) == 0 ){
        usage();
        return( 0 );
      }
      else if ( strcmp( argv[i],"-p" ) == 0 ){
        if ( ++i < argc )
          PORT = strtol( argv[i],NULL,0 );
      }
      else if ( strncmp( argv[i],"-d",2 ) == 0 ){
        DEBUG = 1;
      }
      else if ( strcmp( argv[i],"-nodaemon" ) == 0 ){
        DAEMON = false;
      }
      else{
        usage();
        return( 0 );
      }
    }
  }

#ifdef OS_SUNOS
  // VME init
  openVME24();
  map = mapVME24( V288BASE,msize );
  
  if ( DEBUG ){
    unsigned int  data = vmeread( STATUSREG );
    printf( "caenet : addr=0x%04x data=0x%04x\n",
            STATUSREG,data );
  }
#endif // OS_SUNOS

  // socket init
  struct hostent  *myhost;
  struct sockaddr_in  me;
  //int  sock_listen;
  char  hostname[64];

  gethostname( hostname,64 );
  if ( (myhost = gethostbyname( hostname ) ) == NULL ){
    fprintf( stderr,"Bad hostname : %s\n",hostname );
    perror( "gethostbyname" );
    return( 1 );
  }

  bzero( (char*)&me,sizeof( me ) );
  me.sin_family      = AF_INET;
  me.sin_port        = htons(PORT);
  me.sin_addr.s_addr = ((struct in_addr*)myhost->h_addr)->s_addr;
  
  if ( (sock_listen = socket( AF_INET,SOCK_STREAM,0 ) ) < 0 ){
    fprintf( stderr,"Socket for caenetd failed. sock=%d\n",sock_listen );
    perror( "socket" );
    return( 1 );
  }
  if ( bind( sock_listen, (const sockaddr *)&me, sizeof(me) ) == -1 ){
    fprintf( stderr,"Server caenetd could not bind.\n" );
    perror( "bind" );
    return( 1 );
  }
  // success to bind
  listen( sock_listen, 5 );
  if ( 1|DEBUG )
    printf( "Server started on port %d.\n",PORT );

  //
  //signal(SIGINT, catch_sigint);
  // loop
  while( 1 ){
    sock = accept( sock_listen, NULL, NULL );
    int  pid = (DAEMON) ? fork() : 0;
    if ( pid < 0 ){
      perror( "fork" );
      exit(1);
    }
    else if ( pid == 0 ){
      // Child process
      close( sock_listen );
      process_command( sock );
#ifdef OS_SUNOS
      close( sock );
      unmapVME24( map,msize );
      closeVME24();
#endif // OS_SUNOS
      exit(0);
    }
    else{
      // Parent process
      close( sock );
      // catch SIGCHLD signal to avoid <defunc> processes
      signal(SIGCHLD, SIG_IGN);
    }
  }
  // close
  close( sock_listen );
  close( sock );
  
  // VME terminate
#ifdef OS_SUNOS
  unmapVME24( map,msize );
  closeVME24();
#endif // OS_SUNOS

  return( 0 );
}

