// -*- C++ -*-
//
// caentest.cc
// example code
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Fri May 17 20:40:45 JST 2002
//
// $Id: caentest.cc,v 1.2 2002/09/10 12:12:05 yamaga Exp $
//
// Revision History
//
// $Log: caentest.cc,v $
// Revision 1.2  2002/09/10 12:12:05  yamaga
// updates
//
// Revision 1.1  2002/08/29 14:06:26  yamaga
// Initial version.
//
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>
#ifdef OS_SUNOS
#include <sys/vme.h>
#include <sys/vui.h>
#endif // OS_SUNOS


//////
#define  DEBUG  1

#define  V288BASE  0xff0000

#define  TXBUF         0x0000
#define  RXREG         0x0000
#define  STATUSREG     0x0002
#define  TXREG         0x0004
#define  RESETREG      0x0006
#define  INTERRUPTREG  0x0008

#define  CN_OK         0xfffe
#define  CN_NOOP       0xffff


//////
static int  fd24 = -1;
static unsigned short*  map=0;


//////
int
openVME24()
{
  if ( ( fd24 = open("/dev/vme24d16",O_RDWR) ) < 0 ){
    perror( "userana : open" );
    exit( -1 );
  }
  return( 0 );
}


//////
void
closeVME24()
{
  if ( fd24 >= 0 )
    close( fd24 );
}


//
unsigned short*
mapVME24( off_t addr,unsigned int size )
{
  void*  ptr;
  
  ptr = mmap( 0, size, PROT_WRITE|PROT_READ,MAP_SHARED, fd24, addr );
  if ( ptr == (caddr_t)-1){
    perror("mmap");
    exit(1);
  }

  printf( "mapVME24 : addr=0x%04p size=0x%04x ptr=0x%08p\n",
          addr,size,ptr );

  return( (unsigned short*)ptr );
}


//
void
unmapVME24( void* ptr,unsigned int size )
{
  if ( ptr != (caddr_t)-1 && ptr != 0 ){
    munmap( (char*)ptr,size );
  }
}


//
unsigned short
vmeread( off_t addr )
{
  unsigned short  data;
  
  if ( 0x00 <= addr && addr <= 0x02 ){
    data = map[addr>>1];
    printf( "vmeread : addr=0x%04p data=0x%04x\n",
            addr,data );
  }

  return( data );
}


//
unsigned short
vmewrite( off_t addr, unsigned short cmd )
{
  if ( 0x00 <= addr && addr <= 0x08 ){
    map[addr>>1] = cmd;
    printf( "vmewrite : addr=0x%04p data=0x%04x\n",
            addr,cmd );
  }

  return( 0 );
}


//////
unsigned short
wait_response()
{
  unsigned short  dt,stat;
  do{
    dt   = vmeread( TXBUF );
    stat = vmeread( STATUSREG );
    printf( "wait_response : data=0x%04x stat=0x%04x\n",dt,stat );
  } while( stat==CN_NOOP );
  return( dt );
}


//////
int
read_buf( unsigned short* data, int maxsize )
{
  unsigned short  ret;
  if ( ( ret = wait_response() ) == 0 ){ // success
    unsigned short  dt,stat;
    int  n;
    for( n=0;n<maxsize;n++ ){
      dt   = vmeread( TXBUF );
      stat = vmeread( STATUSREG );
      printf( "read_buf : data=0x%04x stat=0x%04x\n",dt,stat );
      if ( stat==CN_NOOP ) // end of data
        break;
      data[n] = dt;
    }
    return( n );
  }

  printf( "read_buf : wait_response failed : ret=0x%04x\n",ret );
  return( 0 );
}


//////
bool
write_buf( unsigned short data )
{
  vmewrite( TXBUF,data );
  unsigned int  stat;
  do {
    stat = vmeread( STATUSREG );
    printf( "write_buf : data=0x%04x stat=0x%04x\n",data,stat );
  } while( stat!=CN_NOOP && stat!=CN_OK );
  return( (stat==CN_OK) ? true:false );
}


//////
bool
tx_start( unsigned short data )
{
  vmewrite( TXREG,data );
  unsigned int  stat;
  do {
    stat = vmeread( STATUSREG );
    printf( "tx_start : data=0x%04x stat=0x%04x\n",data,stat );
  } while( stat!=CN_NOOP && stat!=CN_OK );
  return( (stat==CN_OK) ? true:false );
}


//////
int
main( int argc, char* argv[] )
{
  int  msize;
  unsigned int  slot=0,channel=0;
  unsigned int  data;
  int  n;
  unsigned short  readbuf[256];

  if ( argc < 3 ){
    printf( "usage : caentest slot ch \n" );
    return( 0 );
  }
  
  slot    = strtol( argv[1],NULL,0 );
  channel = strtol( argv[2],NULL,0 );

#ifdef OS_SUNOS
  openVME24();
  msize = 0x10;
  map = mapVME24( V288BASE,msize );
  
  //
  data = vmeread( STATUSREG );
  printf( "Read : addr=0x%04x data=0x%04x\n",
          STATUSREG,data );

  ////////////////
  // read moduleID
  if ( write_buf( 0x0001 ) == false ) //masterID
    printf( "send failed\n" );
  
  if ( write_buf( 0x0001 ) == false ) // crateNo
    printf( "send failed\n" );
    
  if ( write_buf( 0x0000 ) == false ) // code
    printf( "send failed\n" );
    
  if ( tx_start( 0x0000 ) == false ) // transmisstion start
    printf( "send failed\n" );
    
  //
  if ( ( n = read_buf( readbuf,20 ) ) == 0 ){ // code
    printf( "send failed\n" );
  }
  else{
    char  str[256];
    for( int i=0;i<n;i++ ){
      str[i] = (char) readbuf[i];
      printf( "str[%d]=0x%02x\n",i,(int)str[i] );
    }
    str[n] = 0;
    printf( "moduleID=%s\n",str );
  }

  ////////////////
  // read slotID
  if ( write_buf( 0x0001 ) == false ) //masterID
    printf( "send failed\n" );
  
  if ( write_buf( 0x0001 ) == false ) // crateNo
    printf( "send failed\n" );
    
  if ( write_buf( 0x0003 ) == false ) // code
    printf( "send failed\n" );
    
  if ( write_buf( slot ) == false ) // code
    printf( "send failed\n" );
    
  if ( tx_start( 0x0000 ) == false ) // transmisstion start
    printf( "send failed\n" );
    
  //
  if ( ( n = read_buf( readbuf,20 ) ) == 0 ){ // code
    printf( "send failed\n" );
  }
  else{
    for( int i=0;i<n;i++ ){
      printf( "readbuf[%d]=0x%04x\n",i,readbuf[i] );
    }
    char  str[256];
    strncpy( str,(char*)readbuf,5 );
    str[5] = 0;
    printf( "slot[%d]=%s\n",slot,str );
  }

#if 0
  ////////////////
  // read Channel Data
  if ( write_buf( 0x0001 ) == false ) //masterID
    printf( "send failed\n" );
  
  if ( write_buf( 0x0001 ) == false ) // crateNo
    printf( "send failed\n" );
    
  if ( write_buf( 0x0001 ) == false ) // code
    printf( "send failed\n" );
  
  if ( write_buf( (slot<<4) | channel ) == false ) // code
    printf( "send failed\n" );
  
  if ( tx_start( 0x0000 ) == false ) // transmisstion start
    printf( "send failed\n" );
  
  //
  if ( ( n = read_buf( readbuf,20 ) ) == 0 ){ // code
    printf( "send failed\n" );
  }
  else{
    for( int i=0;i<n;i++ ){
      printf( "readbuf[%d]=0x%04x\n",i,readbuf[i] );
    }
  }
#endif

  
  //
  unmapVME24( map,msize );
  closeVME24();
#endif // OS_SUNOS

  return( 0 );
}

