// -*- C++ -*-
//
// GsimUserMate.cc
// User-defined Material definition function
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Fri Mar 22 16:12:05 JST 2002
//
// $Id: GsimUserMate.cc,v 1.1.1.1 2002/08/13 12:48:08 kensh Exp $
//
// Revision History
//
// $Log: GsimUserMate.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:08  kensh
//
//
// Start version.
//
//

#include <iostream>
#include "ccfortran/ccfortran.h"
#include "ccfortran/G321Common.h"
#include "gsim-general/GsimGeom.h"


//////
// FFkey
float  mfstep = 100.; // steps


//////
void
GsimUserMate()
{
  if ( gcflag_.ISWIT[3-1] > 0 )
    std::cerr << "GsimUserMate" << std::endl; 
  
  // Scintillator, from BELLE library : gsim_ster_mate.F
  float  Ascn[] = {12.01, 1.01};
  float  Zscn[] = { 6,    1   };
  float  Wscn[] = { 1,    1   };
  float  Dscn   = 1.032;
  GSMIXT( n_scintillator,"SCINTIL$", Ascn,Zscn,Dscn, -2, Wscn );

  // CsI(pure), from BELLE library : gsim_ster_mate.F
  float  Acsi[] = {132.91,126.90};
  float  Zcsi[] = { 55,    53   };
  float  Wcsi[] = { 1,    1     };
  float  Dcsi   = 4.53;
  GSMIXT( n_csi,"CsI$", Acsi,Zcsi,Dcsi, -2, Wcsi );

  // Tracking media parameter definition
  int    ISVOL = 0;
  int    IFIELD = 0;   // No magnetic field
  float  FIELDM = 20.; // KGauss
  float  TMAXFD = 30.;
  float  STEMAX = 100.;
  float  DEEMAX = 0.05;
  float  EPSIL  = 0.001;
  float  STMIN  = 0.01;
  GSTMED( n_iron,"IRON",n_iron,ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_tungsten,"TUNGSTEN",n_tungsten,ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_lead,"LEAD",n_lead,ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_air,"AIR",n_air,ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_vacuum,"VACUUM",n_vacuum,ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_scintillator,"SCINTILLATOR",n_scintillator,ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_csi,"CsI",n_csi,ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );

  // With magnetic field (Use IGAUTO=0 to use small step)
  ISVOL  = 0;
  IFIELD = 2;   // Inhomogeneous field : GHELIX
  FIELDM = 20.; // KGauss
  TMAXFD = 30.;
  STEMAX = mfstep; // FFcard : MFST
  DEEMAX = 0.05;
  EPSIL  = 0.001;
  STMIN  = 0.01;
  GSTMED( n_beryllium_mf,"mfBERYLLIUM",n_beryllium, ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_iron_mf,"mfIRON",n_iron,ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_tungsten_mf,"mfTUNGSTEN",n_tungsten, ISVOL,IFIELD, 
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_lead_mf,"mfLEAD",n_lead, ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_air_mf,"mfAIR",n_air,ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );
  GSTMED( n_vacuum_mf,"mfVACUUM",n_vacuum,ISVOL,IFIELD,
          FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0 );

  // dump
  GPRINT( "MATE",0 );
  GPRINT( "TMED",0 );
}
