// -*- C++ -*-
//
// AdcModule.h
//
// Mon Jul 15 09:00:04 JST 2002
// Mitsuhiro YAMAGA
//
// $Id: AdcModule.h,v 1.2 2002/12/01 14:33:09 yamaga Exp $
//
// $Log: AdcModule.h,v $
// Revision 1.2  2002/12/01 14:33:09  yamaga
// Add Id and Log.
//
//
//

#ifndef ADCMODULE_H_INCLDUED
#define ADCMODULE_H_INCLDUED

#include <iostream>
#include <list>

#define  MAX_ADC_CH  128

//
class AdcModule {
public:
  AdcModule( int crate=-1, int pa=-1 );
  ~AdcModule();
  //
  inline int   crate() const { return m_crate; }
  inline int   pa()    const { return m_pa;    }
  inline int   data( int ch ) const ;
  inline const std::list<unsigned int>&  hitList() const { return m_hitList; }
  inline       std::list<unsigned int>&  hitList()       { return m_hitList; }
  inline int   nhit(int threshold=-1 )  const ;
  inline unsigned long long  hitPatternL( int threshold=-1 ) const;
  inline unsigned long long  hitPatternH( int threshold=-1 ) const;
  void         show() const;
  //
  inline void  setCrate( int crate ){ m_crate = crate; }
  inline void  setPa( int pa ) { m_pa = pa; }
  inline int   setData( int ch,int data );
  // friend
  friend std::ostream&  operator<<( std::ostream& out, const AdcModule& m );

private:
  int     m_crate;
  int     m_pa;
  int     m_ch[MAX_ADC_CH];
  std::list<unsigned int>  m_hitList;
};


// inline
//
inline int
AdcModule::nhit( int threshold ) const
{
  int  ct = 0;
  for( int i=0;i<MAX_ADC_CH;i++ ){
    if ( m_ch[i] >= threshold )
      ct++;
  }
  return( ct );
}

//
inline unsigned long long
AdcModule::hitPatternL( int threshold ) const
{
  unsigned long long  pat = 0;
  for( int i=47;i>=0;i-- ){
    pat <<= 1;
    if ( m_ch[i] >= 0 && (m_ch[i] & 0xffff) >= threshold )
      pat |= 1;
  }
  return( pat );
}

//
inline unsigned long long
AdcModule::hitPatternH( int threshold ) const
{
  unsigned long long  pat = 0;
  for( int i=95;i>=48;i-- ){
    pat <<= 1;
    if ( m_ch[i] >= 0 && (m_ch[i] & 0xffff) >= threshold )
      pat |= 1;
  }
  return( pat );
}

//
inline int
AdcModule::data( int ch ) const
{
  if ( 0 <= ch && ch <MAX_ADC_CH )
    return( m_ch[ch] );
  return( -1 );
}


//
inline int
AdcModule::setData( int ch, int data )
{
  if ( 0 <= ch && ch <MAX_ADC_CH )
    return( m_ch[ch] = data );
  return( -1 );
}


#endif // ADCMODULE_H_INCLDUED
